package com.rjconsultores.ventaboletos.web.gui.controladores.catalogos;

import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Textbox;

import com.rjconsultores.ventaboletos.entidad.CoeficienteTarifa;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderCoeficienteTarifa;

@Controller("busquedaCoeficienteTarifaController")
@Scope("prototype")
public class BusquedaCoeficienteTarifaController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	@Autowired
	private transient PagedListWrapper<CoeficienteTarifa> plwCoeficienteTarifa;
	private MyListbox coeficienteTarifaList;
	private Paging pagingCoeficienteTarifa;
	private Textbox txtNombre;

	@Override
	public void doAfterCompose(Component comp) throws Exception {

		super.doAfterCompose(comp);

		coeficienteTarifaList.setItemRenderer(new RenderCoeficienteTarifa());
		coeficienteTarifaList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				CoeficienteTarifa ciu = (CoeficienteTarifa) coeficienteTarifaList.getSelected();
				verCoeficienteTarifa(ciu);
			}
		});

		refreshLista();

		txtNombre.focus();
	}

	private void refreshLista() {
		HibernateSearchObject<CoeficienteTarifa> coeficienteTarifaBusqueda =
				new HibernateSearchObject<CoeficienteTarifa>(CoeficienteTarifa.class, pagingCoeficienteTarifa.getPageSize());

		String nome = txtNombre.getValue();
		if (!nome.isEmpty()) {
			coeficienteTarifaBusqueda.addFilterILike("descCoeficiente", nome.trim().concat("%"));
		}

		coeficienteTarifaBusqueda.addSortAsc("descCoeficiente");
		coeficienteTarifaBusqueda.addFilterEqual("activo", Boolean.TRUE);

		plwCoeficienteTarifa.init(coeficienteTarifaBusqueda, coeficienteTarifaList, pagingCoeficienteTarifa);
		if (coeficienteTarifaList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("busquedaCoeficienteTarifaController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void verCoeficienteTarifa(CoeficienteTarifa coeficienteTarifa) {
		if (coeficienteTarifa == null) {
			return;
		}

		Map args = new HashMap();
		args.put("coeficienteTarifa", coeficienteTarifa);
		args.put("coeficienteTarifaList", coeficienteTarifaList);

		openWindow("/gui/catalogos/editarCoeficienteTarifa.zul",
				Labels.getLabel("editarCoeficienteTarifaController.window.title"), args, MODAL);
	}

	public void onClick$btnPesquisa(Event ev) throws InterruptedException {
		refreshLista();
	}

	public void onClick$btnRefresh(Event ev) {
		refreshLista();
	}

	public void onClick$btnNovo(Event ev) {
		verCoeficienteTarifa(new CoeficienteTarifa());
	}
}
