package com.rjconsultores.ventaboletos.web.gui.controladores.catalogos;

import java.util.Calendar;
import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.Executions;
import org.zkoss.zk.ui.WrongValueException;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Button;
import org.zkoss.zul.Messagebox;

import com.rjconsultores.ventaboletos.entidad.Ciudad;
import com.rjconsultores.ventaboletos.entidad.Escola;
import com.rjconsultores.ventaboletos.service.EscolaService;
import com.rjconsultores.ventaboletos.utilerias.RegistroConDependenciaException;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxCiudad;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextbox;

@Controller("editarEscolaController")
@Scope("prototype")
public class EditarEscolaController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	@Autowired
	private EscolaService EscolaService;
	private Escola escola;
	private MyListbox escolaList;
	private static Logger log = Logger.getLogger(EditarEscolaController.class);
	private MyTextbox txtNome;
	private MyComboboxCiudad cmbCiudad;
	private Button btnApagar;

	public Escola getEscola() {
		return escola;
	}

	public void setEscola(Escola escola) {
		this.escola = escola;
	}

	@Override
	public void doAfterCompose(Component comp) throws Exception {

		super.doAfterCompose(comp);

		escola = (Escola) Executions.getCurrent().getArg().get("escola");
		escolaList = (MyListbox) Executions.getCurrent().getArg().get("escolaList");

		if (escola.getEscolaId() == null) {
			btnApagar.setVisible(Boolean.FALSE);
		} 

		txtNome.focus();
	}

	public void onClick$btnSalvar(Event ev) throws InterruptedException {
		cmbCiudad.getValue();
	
		try {
			escola.setActivo(Boolean.TRUE);
			escola.setFecmodif(Calendar.getInstance().getTime());
			escola.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
			escola.setNombescola(txtNome.getValue());
			
			List<Escola> lsEscola = EscolaService.buscar(escola.getNombescola(), escola.getCiudad());

			boolean podeSalvar = false;
			if (lsEscola.isEmpty()) {
				podeSalvar = true;
			} else {
				if (escola.getEscolaId() == null) {
					podeSalvar = false;
				} else {
					for (Escola e : lsEscola) {
						if (e.getEscolaId().equals(escola.getEscolaId())) {
							podeSalvar = true;
						}
					}
				}
			}
			cmbCiudad.getValue(escola.getCiudad() == null);
			if (podeSalvar) {
				if (escola.getEscolaId() == null) {
					EscolaService.suscribir(escola);
					escolaList.addItem(escola);
				} else {
					EscolaService.actualizacion(escola);
					escolaList.updateItem(escola);
				}
				Messagebox.show(
						Labels.getLabel("editarEscolaController.MSG.suscribirOK"),
						Labels.getLabel("editarEscolaController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);

				closeWindow();
			} else {
				Messagebox.show(
						Labels.getLabel("MSG.Registro.Existe"),
						Labels.getLabel("editarEscolaController.window.title"),
						Messagebox.OK, Messagebox.EXCLAMATION);
			}

		}catch(WrongValueException e){
        	throw e;
        } catch (Exception ex) {
			log.error("editarEscolaController: " + ex);
			Messagebox.show(
					Labels.getLabel("MSG.Error"),
					Labels.getLabel("editarEscolaController.window.title"),
					Messagebox.OK, Messagebox.ERROR);
		}
	}

	public void onClick$btnApagar(Event ev) throws InterruptedException {
		try {
			int resp = Messagebox.show(
					Labels.getLabel("editarEscolaController.MSG.borrarPergunta"),
					Labels.getLabel("editarEscolaController.window.title"),
					Messagebox.YES | Messagebox.NO, Messagebox.QUESTION);

			if (resp == Messagebox.YES) {

				EscolaService.borrar(escola);

				Messagebox.show(
						Labels.getLabel("editarEscolaController.MSG.borrarOK"),
						Labels.getLabel("editarEscolaController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);

				escolaList.removeItem(escola);

				closeWindow();
			}
		} catch (RegistroConDependenciaException ex) {
			Messagebox.show(ex.getLocalizedMessage(), Labels.getLabel("editarEscolaController.window.title"), Messagebox.OK, Messagebox.ERROR);
		}
	}
}
