package com.rjconsultores.ventaboletos.web.gui.controladores.impressaofiscal;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.FiscalRelgerencialEmpresa;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;

@Controller("busquedaRelgerencialEmpresaController")
@Scope("prototype")
public class BusquedaRelgerencialEmpresaController extends MyGenericForwardComposer  {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	@Autowired
    private transient PagedListWrapper<FiscalRelgerencialEmpresa> plwRelgerencialEmpresa;
	private Paging pagingRelgerencialEmpresa;
	
	private Combobox cmbEmpresa;
	private List<Empresa> lsEmpresas;
	
	private MyListbox relgerencialEmpresaList;
	
	@Override
	public void doAfterCompose(Component comp) throws Exception {		
		
		lsEmpresas = UsuarioLogado.getUsuarioLogado().getEmpresa();
		
		super.doAfterCompose(comp);
		
		relgerencialEmpresaList.setItemRenderer(new RelgerencialEmpresaListItemRenderer());
		relgerencialEmpresaList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				FiscalRelgerencialEmpresa relgerencialEmpresa = (FiscalRelgerencialEmpresa) relgerencialEmpresaList.getSelected();
				visualizaRelgerencialEmpresa(relgerencialEmpresa);
			}
		});
		
		actualizaRelgerencialEmpresaList();
	}
	
	private void visualizaRelgerencialEmpresa(FiscalRelgerencialEmpresa relgerencialEmpresa){
		
		if (relgerencialEmpresa == null)
			return;
		
		Map args = new HashMap();
		args.put("relgerencialEmpresa", relgerencialEmpresa);
		args.put("relgerencialEmpresaList", relgerencialEmpresaList);

		openWindow("/gui/impressaofiscal/editarRelgerencialEmpresa.zul",
				Labels.getLabel("editarRelgerencialEmpresaController.window.title"), args, MODAL);
		
	}
	
	public void onClick$btnPesquisa(Event ev) throws InterruptedException {
		actualizaRelgerencialEmpresaList();
	}

	public void onClick$btnRefresh(Event ev) {
		actualizaRelgerencialEmpresaList();
	}

	public void onClick$btnNovo(Event ev) {
		visualizaRelgerencialEmpresa(new FiscalRelgerencialEmpresa());
	}
	
	private void actualizaRelgerencialEmpresaList() {
		HibernateSearchObject<FiscalRelgerencialEmpresa> editarRelgerencialEmpresaBusqueda =
				new HibernateSearchObject<FiscalRelgerencialEmpresa>(FiscalRelgerencialEmpresa.class, pagingRelgerencialEmpresa.getPageSize());

		editarRelgerencialEmpresaBusqueda.addFilterEqual("activo", true);
		
		Comboitem itemEmpresa = cmbEmpresa.getSelectedItem();		
		if (itemEmpresa != null) {
			Empresa empresa = (Empresa) itemEmpresa.getValue();
			editarRelgerencialEmpresaBusqueda.addFilterEqual("empresa", empresa);
		} else {
			editarRelgerencialEmpresaBusqueda.addFilterIn("empresa", lsEmpresas);
		}
		
		plwRelgerencialEmpresa.init(editarRelgerencialEmpresaBusqueda, relgerencialEmpresaList, pagingRelgerencialEmpresa);

		if (relgerencialEmpresaList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("busquedaAidfController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}
	
	public List<Empresa> getLsEmpresas() {
		return lsEmpresas;
	}

	public void setLsEmpresas(List<Empresa> _lsEmpresas) {
		lsEmpresas = _lsEmpresas;
	}
	
}
