package com.rjconsultores.ventaboletos.web.utilerias.api;

import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;

import javax.xml.bind.DatatypeConverter;

import org.apache.http.HttpResponse;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.auth.BasicScheme;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.log4j.Logger;

import com.nimbusds.jose.JOSEException;
import com.nimbusds.jose.JWSAlgorithm;
import com.nimbusds.jose.JWSHeader;
import com.nimbusds.jose.JWSObject;
import com.nimbusds.jose.Payload;
import com.nimbusds.jose.crypto.MACSigner;
import com.nimbusds.jwt.JWTClaimsSet;

public class ApiCallRunnable implements Runnable {

	private static Logger log = Logger.getLogger(ApiCallRunnable.class);
	private static final int minutesExpireToken = 10;
	private static final String secret = "#KO&Fm4_k.sU9M8`6Mx'F\\\"H:*Qxu]6F4r,)JmZ2Jwafd)I.2[RET'1:)VQ6mG9,";
	
	private final String url;
	private final String tenant;
	private String retorno;
	private String urlBase;

	public String getUrl() {
		return url;
	}

	public String getTenant() {
		return tenant;
	}

	public String getRetorno() {
		return retorno;
	}

	public void setRetorno(String retorno) {
		this.retorno = retorno;
	}

	public String getUrlBase() {
		return urlBase;
	}

	public void setUrlBase(String urlBase) {
		this.urlBase = urlBase;
	}

	public ApiCallRunnable(String url, String tenant, String urlOriginal) {
		this.url = url;
		this.tenant = tenant;
		this.urlBase = urlOriginal;
	}

	public void run() {
		HttpClient client = new DefaultHttpClient();
		HttpGet request = new HttpGet(url);
		UsernamePasswordCredentials creds = null;

		try {
			creds = new UsernamePasswordCredentials("internal", getToken());
			request.addHeader(new BasicScheme().authenticate(creds, request));

			if (tenant != null) {
				request.addHeader("x-tenant-id", tenant);
			}

			HttpResponse response = client.execute(request);
			if (response != null && response.getStatusLine() != null) {
				log.info(String.format(":: Response Status Code %d :: Headers = %s", response.getStatusLine().getStatusCode(),Arrays.toString(response.getAllHeaders())));
			}

			Integer codigoRetorno = (response != null && response.getStatusLine() != null) ? response.getStatusLine().getStatusCode() : null;

			retorno = montarMensagemRetorno(tenant, urlBase, codigoRetorno, null);

		} catch (Exception e) {
			retorno = montarMensagemRetorno(tenant, urlBase, null, e.getMessage());
			log.error("Ocorreu um erro ao executar a requisição de limpar cache de localidade da API", e);
		}
	}
	private String montarMensagemRetorno(String tenant, String url, Integer codigoRetorno, String mensagemErro) {

		StringBuilder mensagemRetorno = new StringBuilder();
		mensagemRetorno.append("ULR -> ").append(url).append("\n");
		mensagemRetorno.append("tenant -> ").append(tenant).append("\n");
		if (mensagemErro == null) {
			mensagemRetorno.append("Resposta -> ").append((codigoRetorno != null && codigoRetorno.equals(200)) ? "Solicitação de limpeza de cache executada com sucesso, código " + codigoRetorno : "Erro ao enviar solicitação, código " + codigoRetorno);
		} else {
			mensagemRetorno.append("Resposta -> ").append(mensagemErro);
		}

		return mensagemRetorno.toString();
	}
	private String getToken() throws JOSEException {
		long ttlMillis = minutesExpireToken * 60000;
		long nowMillis = System.currentTimeMillis();
		long expMillis = nowMillis + ttlMillis;

		JWTClaimsSet claims = new JWTClaimsSet.Builder()
				.claim("sub", new SimpleDateFormat("dd/MM/yyyy HH:mm:ss").format(new Date()))
				.claim("userId", "adm")
				.claim("role", "ROLE_TOKEN")
				.claim("exp", expMillis)
				.build();

		JWSObject jwsObject = new JWSObject(new JWSHeader(JWSAlgorithm.HS256),
				new Payload(claims.toJSONObject()));

		jwsObject.sign(new MACSigner(DatatypeConverter.parseBase64Binary(secret)));

		return jwsObject.serialize();
	}

}